#include "Engine/GarbageCollector.h"
#include "Engine/Engine.h"
#include <memory>
#include "Log.h"
#include "Framework/Object.h"

void GarbageCollector::Init()
{
    BaseObject::Init();

    LOG_INFO("Garbage Collector Initialized");
}

void GarbageCollector::CollectGarbage()
{
    // Implement garbage collection logic here
    // Create a iterator to traverse the list of objects and remove unreachable / nullptr objects

    for (auto it = _RootObjects.begin(); it != _RootObjects.end();)
    {
        BaseObject* object = it->get();
        if (!object || object->IsValid())
        {
            LOG_WARN("Removing unreachable object from Garbage Collector: " + object->GetName());
            it = _RootObjects.erase(it);
        }
        else { ++it; }
    }
}

GarbageCollector::GarbageCollector(FluxEngine* engine) : BaseObject(engine)
{
    Init();
}

GarbageCollector::~GarbageCollector()
{
    _RootObjects.clear();
}

void GarbageCollector::AddObject(std::shared_ptr<Object> ObjectToAdd)
{
    if (ObjectToAdd && ObjectToAdd->IsValid())
    {
        _RootObjects.push_back(ObjectToAdd->shared_from_this());
        LOG_INFO("Added object to Garbage Collector: " + ObjectToAdd->GetName());
    }
    else
    {
        LOG_ERROR("Failed to add object to Garbage Collector: " + (ObjectToAdd ? ObjectToAdd->GetName() : "nullptr"));
    }
}

void GarbageCollector::RemoveObject(std::shared_ptr<Object> ObjectToRemove)
{
    if (!ObjectToRemove)
    {
        LOG_ERROR("Failed to remove object from GC: nullptr");
        return;
    }

    auto oldSize = _RootObjects.size();
    auto newEnd = std::remove_if(
        _RootObjects.begin(), _RootObjects.end(),
        [ObjectToRemove](const std::shared_ptr<Object>& obj) {
            return obj.get() == ObjectToRemove.get();
        }
    );

    if (newEnd != _RootObjects.end())
    {
        _RootObjects.erase(newEnd, _RootObjects.end());
        LOG_INFO("Removed object from GC: {0}", object->GetName());
    }
    else
    {
        LOG_ERROR("RemoveObject(): {0} not found in GC",
                  object->GetName());
    }
}
