﻿#include "Log.h"

#include "spdlog/spdlog.h"
#include "spdlog/sinks/stdout_color_sinks.h"

struct LoggerImpl 
{
    LoggerImpl() = default;
    ~LoggerImpl() = default;

    std::shared_ptr<spdlog::logger> _logger;
};

spdlog::logger *Logger::GetLogger() const
{
    return _impl._logger.get();
}

Logger::Logger() : _impl(* new LoggerImpl())
{
    // Initialize the logger
    if (GetLogger()) {
        return;
    }

    // Create a new logger instance
    spdlog::set_pattern("%^[%T] %n: %v%$");

    std::shared_ptr<spdlog::logger> logger = spdlog::get("FLUX");
    if (logger) {
        _impl._logger = logger;
        return;
    }
    
    _impl._logger = spdlog::stdout_color_mt("FLUX");
    _impl._logger->set_level(spdlog::level::trace);

    LogTrace("Logger initialized!");
}

Logger::~Logger()
{
    
}

void Logger::LogTrace(const std::string& message)
{
    if (auto logger = GetLogger())
    {
        logger->trace(message);
    }
}

void Logger::LogInfo(const std::string& message)
{
    if (auto logger = GetLogger())
    {
        logger->info(message);
    }
}

void Logger::LogWarn(const std::string& message)
{
    if (auto logger = GetLogger())
    {
        logger->warn(message);
    }
}

void Logger::LogError(const std::string& message)
{
    if (auto logger = GetLogger())
    {
        logger->error(message);
    }
}

void Logger::LogFatal(const std::string& message)
{
    if (auto logger = GetLogger())
    {
        logger->critical(message);
    }
}
