#pragma once

#include "Core.h"
#include <vector>
#include "Framework/Object.h"
#include "Engine/GarbageCollector.h"

class Logger;

    
class FLUX_API FluxEngine
{
public:
    static FluxEngine* CreateInstance();

    template<typename T, typename... Args>
    static std::unique_ptr<T> CreateApplication(Args&&... args)
    {
        static_assert(std::is_base_of<FluxEngine, T>::value, "T must inherit from FluxEngine");
        std::unique_ptr<T> app = std::make_unique<T>(std::forward<Args>(args)...);
        app->Init();
        return app;
    }

    FluxEngine();
    ~FluxEngine();

    virtual void Init();

    virtual int32_t Start();

    static FluxEngine* GetInstance()
    {
        return GInstance;
    }

    static bool IsRunning()
    {
        return GIsRunning;
    }

    static Logger* GetLogger();

    void RequestExit()
    {
        GIsRunning = false;
    }

    GarbageCollector& GetGarbageCollector();

    void EngineLoop();

private:
    Logger& _Logger;
    GarbageCollector& _GarbageCollector;

    static bool GIsRunning;
    static FluxEngine* GInstance;

    bool bErrorState = false;
};